#!/bin/sh
exec $ZEUSHOME/perl/miniperl -x $0 ${1+"$@"}
 if 0;
#!perl -w

BEGIN { @INC=( "$ENV{ZEUSHOME}/perl/", "$ENV{ZEUSHOME}/webadmin/etc/" ); }
use ZeusOS;
use WebAdmin;
 
print "Pragma: no-cache\n";
print "Cache-Control: no-cache\n";
print "Expires: 0\n";
print "Content-Type: text/html\n\n";


print qq{ <html><head><title>Cluster status summary</title></head>
          <body bgcolor="#ffffff" text="#000000" link="#990066"
          vlink="#990066" alink="#990066">
          <table><tr>
          <td valign=top>
          <img src="/apps/web/title.gif" hspace=10 alt="">
          <p align=right>

          <a href="/apps/admin/docsystem/context.ahtml?help=/apps/web/docs/controller/summary.html" target="help" onClick=
          "window.open('','help','resizable=1,width=640,height=500'); return true;">
          <img src="/apps/web/docs.gif" 
          alt="[documentation]" align=bottom border=0 hspace=0></a>

          <a href="/apps/web/cluster/status.ahtml"><img src="/apps/web/back.gif" 
          alt="[back]" align=bottom border=0 hspace=0></a>
          <a href="/apps/web/"><img src="/apps/web/home.gif" alt="[Controller]"
          align=bottom border=0></a>
          </td><td valign=top><br>
          <img src="title-clusterstatus.gif" alt="Cluster Status">
          <p>
          This page lets you quickly locate any potential problems with your
          cluster setup. Please wait while your cluster is diagnosed...
          </p><br><ul>
        };

# Number of problems found so far
$problems = 0;

# Hash, containing the list of websites each machine is running
%websites = ();

# A list of all backends that are running in non-clustered mode
@nonclustered = ();

# We've got two types of machines
$server[ 0 ] = "frontend load balancer";
$server[ 1 ] = "backend webserver";

$commkey = "$ENV{ZEUSHOME}/webadmin/conf/commkey";

# Check all the backend machines. There should be at least one of these!
$found_backend = 0;
$working_backends = 0;
@real_backends = ();
$nonzeus_backends = 0;


if( open( BACKENDS, "<$ENV{ZEUSHOME}/webadmin/conf/hosts" )) {
  while( <BACKENDS> ) {
    chomp;
    ( $host, $port ) = split /:/;
    $found_backend = 1;
    if( $port == 0 ) { $nonzeus_backends ++ };
    push( @real_backends, $host );
    if( diagnose( 1, $host, $port )) {
      $working_backends ++;

      # Read this backend's configuration (could fail if the backend isn't
      # quite a recent version)
      $conf = get_config( $host, $port );
      # We now have a copy of the backend's global.cfg
      # Look through it and try finding a balancer!enabled line
      if( $conf && ( $conf !~ /balancer!enabled yes/ )) {
         push( @nonclustered, "$host:$port" );
      }
    }
  }
}
 
@real_backends = sort( @real_backends );
$real_backends = "@real_backends ";

# Check all the frontend machines. They should be contactable on their
# control ports. There may in fact be no frontends, in which case the
# following file won't exist:
if( open( LBS, "<$ENV{ZEUSHOME}/webadmin/conf/lbs" )) {
  $working_frontends = 0;
  $frontends = 0;
  while( <LBS> ) {
    chomp;
    ( $host, $port ) = split /:/;
    $frontends ++;
    if( diagnose( 0, $host, $port )) {
      $working_frontends ++;

      # Read the config for this frontend
      $conf = get_config( $host, $port );
      if( $conf ) {
         if( $nonzeus_backends ) {
            # Check that our SSL additions are turned off
            if( $conf !~ /tuning!ssl_send_ip no/ ) {
               warning( "Wrong SSL configuration on $server[ 0 ] $host?", "",
                        "You are using non-Zeus backend webserver(s), however "
                      . "the Zeus SSL logging facilities are still turned on. "
                      . "This will cause problems if your non-Zeus webservers "
                      . "are delivering secure web pages. See the SSL notes on"
                      . " <a href=\"/apps/web/docs/balancer/nonzeus.html\">"
                      . "this page</a> for details." );
            }

            # If you mix Zeus and non-Zeus backends, either you have
            # ssl_send_ip on, in which case the non-Zeus backends fail on SSL.
            # Or, if its turned off, the Zeus backends fail on SSL.
            # Solution is to run the Zeus backends in non-balancer-aware mode.
            if(( $working_backends != $nonzeus_backends ) &&
               ( @nonclustered != $nonzeus_backends )) {
               warning( "Possible problems mixing Zeus and non-Zeus backends",
                        "",
                        "You are using both Zeus and non-Zeus webservers. "
                      . "This is OK, however at least one of your Zeus "
                      . "webserver(s) are running in balancer-aware mode. "
                      . "This will cause problems if SSL pages are delivered "
                      . "by both types of webserver. See the SSL notes on "
                      . "<a href=\"/apps/web/docs/balancer/nonzeus.html\">"
                      . "this page</a> for details." );
            }
         } else {
            # Alternatively, if we're using Zeus backends & they're in
            # clustered mode, then ssl_send_ip should not be turned off
            if(( @nonclustered == 0 ) &&
               ( $conf =~ /tuning!ssl_send_ip no/ )) {
               warning( "Wrong SSL configuration of $server[ 0 ] $host?", "",
                        "The Zeus SSL logging facilities have been turned off "
                      . "on this Load Balancer. However, you are using Zeus "
                      . "backend webservers, which expect the logging. You "
                      . "must edit the file <tt>\$ZEUSHOME/balancer/global.cfg"
                      . "</tt> on this machine and remove the line '<tt>"
                      . "tuning!ssl_send_ip no</tt>'. Then restart the "
                      . "Load Balancer." );
            }
         }
      }
      
      # Query this frontend and find out which backends it knows about
      my $error = "";
      my $hosts = "";
      my $result = WebAdmin::command( $host, $port, "cells", $error, 15 );
      $result ||= "Failure\n$error";
      my( $status, $data ) = split( '\n', $result, 2 );
      if( $status eq "Success" ) {
        my @backend_list = map( { /(.*?):/ } split( '\n', $data ));

        # Check that the two lists of backends match
        $warn = "";

        # Common case will be that someone forgot to copy over the 'workers'
        # file, so the balancer has no backend workers at all
        if( @backend_list == 0 ) {
           $title = "Empty 'workers' file on $server[ 0 ] $host";
           $warn = "This Load Balancer has not been configured to use any " .
                   "backend webservers!";
        } else {
           # Try and work out which backends differ - generate 2 lists; those
           # on frontend, those on admin
           compare_lists( \@backend_list, \@real_backends );
           if(( @diff1 > 0 ) || ( @diff2 > 0 )) {
              $title = "Wrong 'workers' file on $server[ 0 ] $host";
              $warn = "This Load Balancer is configured to use different " .
                      "backend webservers from those set up in the Admin " .
                      "server.";
           }

           if( @diff1 > 0 ) {
              $warn .= "<p>This frontend balancer does not know about the " .
                       "following backend server(s):</p><ul>";
              foreach $backend ( @diff1 ) {
                 $warn .= "<li>$backend</li>";
              }
              $warn .= "</ul>";
           }

           if( @diff2 > 0 ) {
              $warn .= "<p>This frontend balancer lists the following " .
                       "backend server(s) which are unknown to the Admin " .
                       "Server:</p><ul>";
              foreach $backend ( @diff2 ) {
                 $warn .= "<li>$backend</li>";
              }
              $warn .= "</ul>";
           }
        }

        if( $warn ne "" ) {
           warning( $title, "", $warn . "<p>You have probably " .
                    "forgotten to copy across the file " .
                    "$ENV{ZEUSHOME}/webadmin/conf/hosts from the Admin " .
                    "Server into \$ZEUSHOME/balancer/workers on this " .
                    "Balancer." );
# Commented out because we dont yet support copying files across...
#           warning( $title, "", $warn . "<p>" .
#                    "<a href=\"synclb.cgi?server=$host:$port\">" .
#                    "Click here</a> to send the Load Balancer the correct " .
#                    "configuration. " );
        }

      } else {
        ( $_, $error ) = split( '\n', $result );
        if( $error =~ /^Invalid\ command/ ) {
          $warn = "Check that you have not confused backend and frontend " .
                  "machines in your configuration. This Load Balancer " .
                  "did not understand a Load Balancer query command.";
        } else {
          $warn = "The frontend machine may not be running correctly. " .
                  "Please restart this machine and try again." ;
        }
        warning( "Could not query $server[ 0 ] $host", $data, $warn );
      }
    }
  }
  close( LBS );

  if( $frontends > 2 ) {
    warning( "There are $frontends frontend machines", "",
             "The Load Balancer supports a maximum of two frontend " .
             "machines, more frontends are unnecessary!" );
  }
  if(( $frontends > 0 ) && ( $working_frontends == 0 )) {
    warning( "There are no working frontend load balancers for this cluster",
             "",
             "If there are no working frontends, you will not " .
             "be able to deliver webpages!" );
  }

  # If we have a frontend, then all backends must be set up in 'clustered' mode
  if( $frontends && @nonclustered ) {
     foreach $machine ( @nonclustered ) {
        warning( "Backend machine $machine misconfigured", "",
                 "Backend machine $machine is not running in 'balancer-aware' "
               . "mode. This means that it will not work correctly behind "
               . "a Load Balancer. To correct this, you must add the line "
               . "'<tt>balancer!enabled yes</tt>' to the configuration file "
               . "<tt>\$ZEUSHOME/web/global.cfg</tt> on the backend machine, "
               . "and then restart the server." );
     }
  }
}

if( !$found_backend ) {
  warning( "There were no backend webservers found", "",
           "Your cluster needs at least one webserver, else you will not " .
           "be able to deliver webpages!" );
} elsif( $working_backends == 0 ) {
  warning( "There are no working backend webservers for this cluster", "",
           "Your cluster needs at least one webserver, else you will not " .
           "be able to deliver webpages!" );
}

# Check that all running machines are hosting the same websites
if(( $host, $sites ) = each %websites ) {
  # Find out what websites _should_ be running
  my $dir = "$ENV{ZEUSHOME}/webadmin/conf/runningsites";
  if( opendir( RUNNINGSITES, "$dir" )) {
    @realsites = sort( grep !/^\./, readdir RUNNINGSITES );
    closedir( RUNNINGSITES );

    do {
      @hostsites = sort( split( ' ', $sites ) );
      # Get a list of the sites that the adminserver is lacking and the
      # server is lacking
      compare_lists( \@hostsites, \@realsites );
      $warn = "";

      if(( @diff1 > 0 ) || ( @diff2 > 0 )) {
         $title = "Server '$host' isn't running the correct virtual server(s)";
         $warn = "This server is configured to be running different virtual " .
                 "servers from those set up in the Admin server.";
      }

      if( @diff1 > 0 ) {
         $warn .= "<p>This server is not running the following virtual " .
                  "server(s):</p><ul>";
         foreach $site ( @diff1 ) { $warn .= "<li>$site</li>"; }
         $warn .= "</ul>";
      }

      if( @diff2 > 0 ) {
         $warn .= "<p>This server is running the following virtual " .
                  "server(s) which, according to the Admin server, should " .
                  "not be running:</p><ul>";
         foreach $site ( @diff2 ) { $warn .= "<li>$site</li>"; }
         $warn .= "</ul>";
      }

      if( $warn ne "" ) {
         warning( $title, "", $warn . "<p>Stop and start the affected " .
                  "servers, to send the right configuration to '$host'. " .
                  "<a href=\"sync.cgi?server=$host:$port\">" .
                  "Click here</a> to do this now." );
      }

    } while(( $host, $sites ) = each %websites );
  } else {
    warning( "Can't access $dir", "Error: $!",
             "The Admin Server needs to access this directory in order to " .
             "read website configurations. Please fix this error, then " .
             "reload this page." );
    exit( 0 );
  }
}

if( $problems == 0 ) {
  print "<h3>No problems were found with the cluster.</h3>";
}
finished();

# TODO:
# Contact each machine on each listening port?
# Try fetching / from each website?
# Make website handling a bit smarter... e.g. for cases where theres hundreds
# of running websites.


# Compare two lists (of websites/servers), and setup two lists; those elements
# in list 2 that aren't in list 1, and vv. 
sub compare_lists( \@\@ ) {
   my( $list1, $list2 ) = @_;
   @diff1 = ();
   @diff2 = ();
   my $string1 = " @$list1 ";
   my $string2 = " @$list2 ";

   foreach $item ( @$list1 ) {
      if( $string2 !~ /\ $item\ / ) { push( @diff2, $item ); }
   }

   foreach $item ( @$list2 ) {
      if( $string1 !~ /\ $item\ / ) { push( @diff1, $item ); }
   }
}


# Finished diagnosis. End the page
sub finished() {
  print "</ul>\n<br>Diagnosis finished.</td></tr></table></body></html>";
  exit(0);
}


# Diagnose a machine. Returns 1 if the machine has no problems. Otherwise,
# it outputs HTML reporting the problem.
sub diagnose( $$$ ) {
  my( $type, $host, $port ) = @_;
  my $error = "";
  if( $port == 0 ) {
     # Yep, using a Non-Zeus webserver is definitely an error :)
     diagnose_error( $type, $host, $port, "Non-Zeus server" );
     return 1;
  }
  $result = WebAdmin::command( $host, $port, "info", $error, 5 );
  if( $result ) {
    ( $success, $error ) = split( '\n', $result, 2 );
    if( $success eq "Success" ) {
      # Remember which websites this machine thinks its running
      $websites{ $host } = $error;
      return 1;
    }
  } else {
    chomp $error;
  }
  diagnose_error( $type, $host, $port, $error );
  return 0;
}


# Grab the global.cfg file of a remote machine
# Returns undefined if it failed, or the machine is non-Zeus
sub get_config( $$ ) {
   my( $host, $port ) = @_;
   my $error = "";
   my $config;
   if( $port ) {
      $result = WebAdmin::command( $host, $port, "getconfig", $error, 15 );
      $result ||= "Failure\n$error";
      ( $success, $config ) = split( '\n', $result, 2 );
      if( $success ne "Success" ) { $config = 0 };
   }
   return $config;
}


# Print a warning message to the html
sub warning( $$$ ) {
  my( $title, $error, $text ) = @_;
  print "<li><h3>$title</h3><p>";
  print "<h4>$error</h4>" unless( "$error" eq "" );
  print "$text</p><br></li>\n";
  $problems ++;
}


# Diagnose whats wrong with a machine using the error message from command()
sub diagnose_error( $$$$ ) {
  my( $type, $host, $port, $error ) = @_;
  my $title = $server[ $type ];
  my $message;
  if( $error =~ /No\ route\ to\ host/ ) {
    $message = "The Admin Server was unable to contact $host. Check that " .
      "your networking is set up correctly. Check that '$host' is alive!";
  } elsif( $error =~ /^connect/ ) {
    $message = "It is likely that the $title is not running on $host, " .
      "or is configured to listen on the wrong port. According to the " .
      "Admin Server, $host should have its control port set to $port.";
  } elsif( $error =~ /^(socket|syswrite)/ ) {
    $message = "A network error prevented the diagnostic tool from " .
      "contacting $host.";
  } elsif( $error =~ /^Unable\ to\ open\ communication/ ) {
    $message = "The communication key, which is required in order to " .
      "communicate with frontend and backend machines, was not found. Check " .
      "that the Admin Server can read the file $commkey"
  } elsif( $error =~ /^Unable\ to\ resolve/ ) {
    $message = "Check that '$host' is the right address for this $title. " .
      "If so, check that the DNS is configured correctly on the " .
      "Admin Server and each frontend machine.";
  } elsif( $error =~ /^operation\ timeout/ ) {
    $message = "$host did not respond to the status request in time. " .
      "This may have been caused by poor network connectivity, or perhaps " .
      "the $title is not running properly. Check the network or try " .
      "restarting $title.";
  } elsif( $error =~ /^Non-Zeus/ ) {
    $message = "$host is not a Zeus server. Many of the diagnostic tests " .
      "cannot be performed on servers which lack the functionality of the " .
      "Zeus Web Server, so you must check for yourself that this server " .
      "is running correctly!";
  } elsif( $error =~ /Invalid\ communication\ key/ ) {
    $message = "$host does not have the correct communication key. " .
      "Copy the file $commkey from the Admin Server into the " .
      "directory \$ZEUSHOME/";
    if( $type == 0 ) { $message .= "balancer/etc/"; }
    else { $message .= "web/etc/"; }
    $message .= "on $host, then restart the $title.";
  } else {
    $message = "";
  }
  warning( "Problem with $title $host:$port", $error, $message );
}
