#!/bin/sh
exec $ZEUSHOME/perl/miniperl -x $0 ${1+"$@"}
 if 0;
#!perl -w

BEGIN { @INC=( "$ENV{ZEUSHOME}/perl/", "$ENV{ZEUSHOME}/webadmin/etc/" ); }
use ZeusOS;
use WebAdmin;

# Args:
#
# This rather complex heirarchy of args allows us to drill down
# through the data levels 'machine', 'virtual server', 'hheader'
#
# machine = name|_sum|_each
# vserver = name|_sum|_top5|_top10|''
# hheader = name|_sum|_top5|_top10|''
#
# refine  = hheader|vserver|machine|no
#
# time    = 24hours|7days
# value   = hits|bytes
#
# output  = text/html|image/gif

# default values

%default_arg = (
   'machine' => '_sum',
   'vserver' => '',
   'hheader' => '',
   'refine'  => 'no',
   'time'    => '24hours',
   'value'   => 'bytes',
   'output'  => 'text/html'
);

@cols = (
   "3333cc", "33cc33", "cc3333", 
   "339999", "993399", "999933",
   "666666", 
   "9999cc", "99cc99", "cc9999",
   "66cccc", "cc66cc", "cccc66",
   "999999",
   );
   
@days = ('Sun', 'Mon', 'Tue', 'Wed', 'Thur', 'Fri', 'Sat');

&parse_form_data(*arg);

# Get system configuration

open IN, "$ENV{'ZEUSHOME'}/webadmin/conf/hosts" or die;
@machines = map { chomp; $_ } <IN>;
close IN;
opendir IN, "$ENV{'ZEUSHOME'}/webadmin/conf/sites" or die;
@vservers = sort grep {-f "$ENV{'ZEUSHOME'}/webadmin/conf/sites/$_"} readdir IN;
closedir IN;

$now = time;

# Sanify args

foreach (grep /^action-/, keys %arg) {
   /^action-(.*?)-(.*)$/; $arg{$1} = $2;
}

foreach (keys %default_arg) {
   $arg{$_} = $default_arg{$_} if (!defined $arg{$_});
}

$arg{'machine'} = "_sum" if ($arg{'machine'} eq "_line");
$arg{'vserver'} = "_sum" if ($arg{'vserver'} eq "_line");

$arg{'machine'} = $machines[0] if (scalar(@machines) == 1);

$arg{'hheader'} = $arg{'hheader-named'} if ($arg{'hheader-named'});

$hopeful = 0;
if ($arg{'refine'} eq 'machine') {
   if ((!$arg{'machine'}) || ($arg{'machine'} eq '_each')) {
      $arg{'machine'} = '_sum';
   }
   $arg{'vserver'} = '';
   $arg{'hheader'} = '';
}
elsif ($arg{'refine'} eq 'vserver') {
   if ((!$arg{'machine'}) || ($arg{'machine'} eq '_each')) {
      $arg{'machine'} = '_sum';
   }
   if (!$arg{'vserver'}) {
      $arg{'vserver'} = ($arg{'machine'} eq "_sum")?"_sum":"_top5";
      $hopeful = 1;
   }
   $arg{'hheader'} = '';
}
elsif ($arg{'refine'} eq 'hheader') {
   if ((!$arg{'machine'}) || ($arg{'machine'} eq '_each')) {
      $arg{'machine'} = '_sum';
   }
   
   
   if ((!$arg{'vserver'}) || ($arg{'vserver'} =~ /^_top[0-9]+$/)) {
      $arg{'vserver'} = '_sum';
   }
   if (!$arg{'hheader'} || ($arg{'hheader'} !~ /^_/)) {
      $arg{'hheader'} = ($arg{'machine'} eq "_sum")?"_sum":"_top5";
      $hopeful = 1;
   }
}

if ( ($arg{'machine'} eq "_sum") && 
     (($arg{'vserver'} =~ /^_top[0-9]+$/) || ($arg{'hheader'} =~ /^_top[0-9]+$/))
   ) {
   Error("You can't calculate the 'top <n> values'\non more than one machine at a time");
}

if ( ($arg{'vserver'} =~ /^_top[0-9]+$/) && ($arg{'hheader'})) {
   Error("You can't list host headers for\nthis list ($arg{'vserver'}) of virtual servers");
}

if ( ! defined $arg{'back'} ) {
   $arg{'back'} = '/apps/web';
}

# Generate a unique string for hashing the query:
$uniq = "$arg{'machine'};$arg{'vserver'};$arg{'hheader'};$arg{'time'};$arg{'value'};$arg{'back'};".int(time/150);
$hash = hash($uniq);

if (-f "./cache/graph.$hash.html" && -f "./cache/graph.$hash.gif" &&
    -M "./cache/graph.$hash.html" <0.0005 ) {
   open IN, "<./cache/graph.$hash.html";
   read (IN, $html, (-s "./cache/graph.$hash.html"));
   close IN;
   
   print_html($html);
   exit 0;
}

# Generate HTML standard text

# Outer table - banner on left, stats on right
$html = qq(
<table border=0>
<tr>
<td valign=top>
<img src="title.gif" hspace=10 alt="">
<p align=right>
__back_doc__
</td>
);


$back = "";
if ((defined $arg{'back'}) && $arg{'back'}) {
   $otherargs .= "&back=".url_esc($arg{'back'});
   $back = "<input type=hidden name=back value=\"$arg{'back'}\">";
}

# back & docs buttons
if ($back) {
  $back_doc = qq(
    <a href="/apps/admin/docsystem/context.ahtml?help=/apps/web/docs/controller/stats.html" target=help onClick="window.open('','help','resizable=1,width=640,height=500'); return true;">
    <img src="/apps/web/docs.gif" alt="[documentation]" border=0></a>
<a href="$arg{'back'}"><img src="/apps/web/back.gif" alt="[back]" border=0></a>
    );
}

$html =~ s/__back_doc__/$back_doc/m;

# graph etc cell
$html .= qq(

<td>
<table cellspacing=4 cellpadding=0>
<tr><td align=center>
<img src="title-activity.gif" alt="Activity">
<form action="graph.cgi" method="GET">
);

$otherargs = "time=$arg{'time'}\&value=$arg{'value'}";

($bybytes, $byhits )=($arg{'value'} eq "bytes" )?("selected",""):("","selected");
($by24hrs, $by7days)=($arg{'time'} eq "24hours")?("selected",""):("","selected");

$params = qq(
Show
<select name=value>
<option value="bytes" $bybytes>Bytes</option>
<option value="hits"  $byhits >Hits </option>
</select>
for the previous
<select name=time>
<option value="24hours" $by24hrs>24 hours</option>
<option value="7days"   $by7days>7 days  </option>
</select>
$back
);

if (!$arg{'vserver'}) {
   # plot by machine
    
    my $title = "Traffic analysis for ";
    if   ( $arg{'machine'} =~ /_sum/ )  { $title .= "all clustered machines"; }
    elsif( $arg{'machine'} =~ /_each/ ) { $title .= "each clustered machine"; }
    else                                { $title .= &prettify_machinename( $arg{'machine'} ) };

    $arg{'time'} =~ m/(\d+)(.*)/;
    my $subtitle = "($arg{'value'} for last $1 $2)";


   ($res, $msg, @vals) = statd_query_1(
      $arg{'machine'}, "*", "", $arg{'value'}, $arg{'time'});

   if (!$res) {
      Error("The plot failed, with the following error messages:\n".$msg);
   }
   
   ($file, $t) = plot_graph(
      $res, "Traffic Analysis", $arg{'value'}, $arg{'time'}, 
      "machine=!!!!\&$otherargs", @vals);
   
   $warnings = ($msg)?"<hr><b>Warning:</b><blockquote><pre>$msg</pre></blockquote>\n":"";
   
   @options = (((@machines > 1)?("_sum", "_each", "_line"):()), @machines);

   $machines_opt = join "\n", 
      map { "<option value=\"$_\" ".
             (($_ eq $arg{'machine'})?"selected":"").
            ">".prettify(prettify_machinename($_))."</option>" } @options;
         
   $html .= qq(

<font size=+3><!title>$title<!/title></font>
<br><font size=+1>$subtitle</font><p>
<img src="$file" width=450 height=300>
$t
</td></tr><tr><td>
$warnings
<hr>
<center><table cellspacing=0 cellpadding=2><tr><td>
<li> Machine: 
<select name=machine>
$machines_opt</select>

<br>
<li> $params

<br>
<input type=submit name="action-plot-yes"       value="Plot">
<input type=submit name="action-refine-vserver" value="-&gt; Virtual Servers">
<input type=submit name="action-refine-hheader" value="-&gt; Host Headers">
</td></tr></table></center>
);
} 
elsif (!$arg{'hheader'}) {
   # plot by vserver

    my $title = "Traffic analysis for ";
    if   ( $arg{'vserver'} =~ /_sum/ ) { $title .= "all virtual servers "; }
    elsif( $arg{'vserver'} =~ /_top(\d+)/ ) { $title .= "top $1 virtual servers (by hits) "; }
    else  { $title .= "virtual server $arg{'vserver'} "; }

    if( $arg{'machine'} =~ /_sum/ ) { $title.= "on all machines "; }
    else { $title .= "on machine ". &prettify_machinename( $arg{'machine'} ). " "; }

    $arg{'time'} =~ m/(\d+)(.*)/;
    my $subtitle = "($arg{'value'} for last $1 $2)";
    

   ($res, $msg, @vals) = statd_query_2(
      $arg{'machine'}, $arg{'vserver'}, "", $arg{'value'}, $arg{'time'});

   if (!$res) {
      Error("The plot failed, with the following error messages:\n".$msg);
   }
   
   ($file, $t) = plot_graph(
      $res, "Traffic Analysis by Virtual Server", $arg{'value'}, $arg{'time'}, 
      "machine=$arg{'machine'}\&vserver=!!!!\&$otherargs", @vals);
   
   $warnings = ($msg)?"<hr><b>Warning:</b><blockquote><pre>$msg</pre></blockquote>\n":"";

   @options = ("_sum", (($arg{'machine'} ne "_sum")?("_top5", "_top10", "_top15", "_top20"):()), "_line", @vservers);
   
   $snapshot = "";
   if ($arg{'machine'} ne "_sum") {
      $snapshot = "&nbsp;&nbsp;(<a href=\"snapshot.cgi?machine=$arg{'machine'}&value=$arg{'value'}\">5 minute snapshot</a>)";
   }
   
   $vservers_opt = join "\n", 
      map { "<option value=\"$_\" ".
             (($_ eq $arg{'vserver'})?"selected":"").
            ">".prettify($_)."</option>" } @options;

   $_mc = prettify_3($arg{'machine'});

   $html .= qq(
<font size=+3><!title>$title<!/title></font>
<br><font size=+1>$subtitle</font><p>
<img src="$file" width=450 height=300>
$t
</td></tr><tr><td>
$warnings
<hr>
<center><table cellspacing=0 cellpadding=2><tr><td>
Machine:
<input type=hidden name=machine value=$arg{'machine'}>
<b>$_mc</b>;&nbsp;

<br>
<li> Virtual Server: <select name=vserver>
$vservers_opt</select>

<br>
<li> $params

<br>
<input type=submit name="action-plot-yes"       value="Plot">
<input type=submit name="action-refine-machine" value="&lt;- Machines">
<input type=submit name="action-refine-hheader" value="-&gt; Host Headers">
$snapshot
</td></tr></table></center>
);
} 
elsif (($arg{'hheader'} eq "_sum") || ($arg{'hheader'} =~ /^_top[0-9]+/)) {
   # plot by hheader (unnamed)

    my $title = "Traffic analysis for ";
    if   ( $arg{'hheader'} =~ /_sum/ ) { $title .= "all host headers "; }
    else { $arg{'hheader'} =~ /_top(\d+)/; $title .= "top $1 host headers (by hits) "; }

    if   ( $arg{'vserver'} =~ /_sum/ ) { $title .= "on all virtual servers "; }
    else  { $title .= "on virtual server $arg{'vserver'} "; }

    if( $arg{'machine'} =~ /_sum/ ) { $title.= "on all machines "; }
    else { $title .= "on machine ". &prettify_machinename( $arg{'machine'} ). " "; }

    $arg{'time'} =~ m/(\d+)(.*)/;
    my $subtitle = "($arg{'value'} for last $1 $2)";

    
   ($res, $msg, @vals) = statd_query_3(
      $arg{'machine'}, $arg{'vserver'}, $arg{'hheader'}, $arg{'value'}, $arg{'time'});

   if (!$res) {
      Error("The plot failed, with the following error messages:\n".$msg);
   }
   
   ($file, $t) = plot_graph(
      $res, "Traffic Analysis by Host Header", $arg{'value'}, $arg{'time'}, 
      "machine=$arg{'machine'}\&vserver=$arg{'vserver'}\&hheader=!!!!\&$otherargs", @vals);
   
   $warnings = ($msg)?"<hr><b>Warning:</b><blockquote><pre>$msg</pre></blockquote>\n":"";
   
   @options = ("_sum", ($arg{'machine'} ne "_sum")?("_top5", "_top10", "_top15", "_top20"):());

   $snapshot = "";
   if ($arg{'machine'} ne "_sum") {
      $snapshot = "&nbsp;&nbsp;(<a href=\"snapshot.cgi?machine=$arg{'machine'}&vserver=$arg{'vserver'}&value=$arg{'value'}\">5 minute snapshot</a>)";
   }
   
   $hheaders_opt = join "\n", 
      map { "<option value=\"$_\" ".
             (($_ eq $arg{'hheader'})?"selected":"").
            ">".prettify($_)."</option>" } @options;

   $_mc = prettify_3($arg{'machine'});
   $_vs = prettify_3($arg{'vserver'});
         
   $html .= qq(
<font size=+3><!title>$title<!/title></font>
<br><font size=+1>$subtitle</font><p>
<img src="$file" width=450 height=300>
$t
</td></tr><tr><td>
$warnings
<hr>
<center><table cellspacing=0 cellpadding=2><tr><td>
Machine:
<input type=hidden name=machine value=$arg{'machine'}>
<b>$_mc</b>;&nbsp;

Virtual Server:
<input type=hidden name=vserver value=$arg{'vserver'}>
<b>$_vs</b>;&nbsp;

<br>
<li> Host Header:
<select name=hheader>
$hheaders_opt</select>
<i>or</i>&nbsp;&nbsp;
<input type=text name="hheader-named" value="" size=18>

<br>
<li> $params

<br>
<input type=submit name="action-plot-yes"       value="Plot">
<input type=submit name="action-refine-vserver" value="&lt;- Virtual Servers">
<input type=submit name="action-refine-machine" value="&lt;- Machines">
$snapshot
</td></tr></table></center>
);
}
else {
   # plot by named hheader

    my $title = "Traffic analysis for $arg{'hheader'} ";

    if   ( $arg{'vserver'} =~ /_sum/ ) { $title .= "on all virtual servers "; }
    else  { $title .= "on virtual server $arg{'vserver'} "; }

    if( $arg{'machine'} =~ /_sum/ ) { $title.= "on all machines "; }
    else { $title .= "on machine ". &prettify_machinename( $arg{'machine'} ). " "; }

    $arg{'time'} =~ m/(\d+)(.*)/;
    my $subtitle = "($arg{'value'} for last $1 $2)";


   ($res, $msg, @vals) = statd_query_4(
      $arg{'machine'}, $arg{'vserver'}, $arg{'hheader'}, $arg{'value'}, $arg{'time'});

   if (!$res) {
      Error("The plot failed, with the following error messages:\n".$msg);
   }
   
   ($file, $t) = plot_graph(
      "$res", "$vals[0]", $arg{'value'}, $arg{'time'}, 
      "machine=$arg{'machine'}\&vserver=$arg{'vserver'}\&hheader=!!!!\&$otherargs", @vals);
   
   $warnings = ($msg)?"<hr><b>Warning:</b><blockquote><pre>$msg</pre></blockquote>\n":"";
   
   $_mc = prettify_3($arg{'machine'});
   $_vs = prettify_3($arg{'vserver'});

   $html .= qq(
<font size=+3><!title>$title<!/title></font>
<br><font size=+1>$subtitle</font><p>
<img src="$file" width=450 height=300>
$t
</td></tr><tr><td>
$warnings
<hr>
<center><table cellspacing=0 cellpadding=2><tr><td>
Machine:
<input type=hidden name=machine value=$arg{'machine'}>
<b>$_mc</b>;&nbsp;

Virtual Server:
<input type=hidden name=vserver value=$arg{'vserver'}>
<b>$_vs</b>;&nbsp;

Host Header:
<input type=hidden name=hheader value=$arg{'hheader'}>
<b>$arg{'hheader'}</b>;&nbsp;

<br>
<li> $params

<br>
<input type=submit name="action-plot-yes"       value="Plot">
<input type=submit name="action-refine-hheader" value="&lt;- Host Headers">
<input type=submit name="action-refine-vserver" value="&lt;- Virtual Servers">
<input type=submit name="action-refine-machine" value="&lt;- Machines">
</td></tr></table></center>
);
}

$html .= qq(
<hr>
</form>
);

$html .= qq(
</td></tr></table></td></tr></table>
);

open OUT, ">./cache/graph.$hash.html";
print OUT $html;
close OUT;

print_html($html);

exit 0;

sub statd_query_1 {
   my($machine, $vserv, $hhead, $value, $time) = @_;
   my($response, $msg) = ("", "");
   
   if ($machine ne "_each") {
      ($response, $msg) = statd_query_machines($machine, "Q",
         "$value\t$vserv\t$hhead\t0\t".statd_time($time));

      return ($response, $msg, $machine);
   } else {
      $response = $msg = "";
      my(%response) = ();
      foreach $mach (@machines) {
         my ($_response, $_msg) = statd_query_machines($mach, "Q",
            "$value\t$vserv\t$hhead\t0\t".statd_time($time));
         
         $msg .= $_msg;
         # Decode response...
         my ($h, $b) = split "\n", $_response, 2;
         if (defined ($b)) {
            $response = $h."\n" if (!$response);

            chomp($b); chomp($b);
            $b =~ /\n([0-9]+) /;
            $response{$mach}{'max'} = $1;
            $response{$mach}{'response'} = $b;
         }
      }
      my(@mach) = ();
      foreach $mach (sort 
            {$response{$b}{'max'} <=> $response{$a}{'max'}}
            keys %response) {
         $response .= $response{$mach}{'response'}."\n";
         push @mach, $mach;
      }
      
      $response =~ s/[0-9]+/scalar(@mach)/e;

      return ($response, $msg, @mach);
   }
}

sub statd_query_2 {
   my($machine, $vserv, $hhead, $value, $time) = @_;
   my($response, $msg, $msg2) = ("", "", "");

   my (@vserv);

   if ($vserv =~ /^_top([0-9]+)$/) {
      my ($n) = $1;
      # need to get list of top $n vservers
      
      ($response, $msg) = statd_query_machines($machine, "R",
         "$value\t*\t\t$n\t".statd_time($time));
      
      my ($h, $t, @vals) = split /\n/, $response;
      
      @vserv = grep /./, map { /^.*?\t.*?\t(.*?)\s.*$/?$1:"" } @vals;
      
      if (@vserv) {
         $_vserv = join " ", @vserv;
      } else {
         # Oh-oh... no summary file!
         $arg{'vserver'} = "_sum";
         @vserv = ("_sum");
         $_vserv = "*";
      }
   } elsif ($vserv eq "_sum") {
      @vserv = ($vserv);
      $_vserv = "*";
   } else {
      @vserv = ($vserv);
      $_vserv = $vserv;
   }
   ($response, $msg2) = statd_query_machines($machine, "Q",
      "$value\t$_vserv\t$hhead\t0\t".statd_time($time));

   return ($response, $msg.$msg2, @vserv);
}

sub statd_query_3 {
   my($machine, $vserv, $hhead, $value, $time) = @_;
   my($response, $msg, $msg2) = ("", "", "");

   my ($_vserv) = ($vserv eq "_sum")?"*":$vserv;
   my (@hhead);
   my ($_hhead);

   if ($hhead =~ /^_top([0-9]+)$/) {
      my ($n) = $1;
      # need to get list of top $n host header lines

      ($response, $msg) = statd_query_machines($machine, "R",
         "$value\t$_vserv\t*\t$n\t".statd_time($time));

      my ($h, $t, @vals) = split /\n/, $response;
      
      @hhead = grep /./, map { /^.*?\t(.*?)\t.*$/?$1:"" } @vals;
      
      if (@hhead) {
         $_hhead = join " ", @hhead;
      } else {
         # Oh-oh... no summary file!
         $arg{'hheader'} = "_sum";
         @hhead  = ("_sum");
         $_hhead = "*";
      }
   } elsif ($hhead eq "_sum") {
      @hhead = ($hhead);
      $_hhead = "*";
   } else {
      @hhead = ($hhead);
      $_hhead = $hhead;
   }
   
   ($response, $msg2) = statd_query_machines($machine, "Q",
      "$value\t$_vserv\t$_hhead\t0\t".statd_time($time));

   return ($response, $msg.$msg2, @hhead);
}

sub statd_query_4 {
   my($machine, $vserv, $hhead, $value, $time) = @_;
   my($response, $msg) = ("", "");

   my ($_vserv) = ($vserv eq "_sum")?"*":$vserv;
   
   # $hhead named
   ($response, $msg) = statd_query_machines($machine, "Q",
      "$value\t$_vserv\t$hhead\t0\t".statd_time($time));
   return ($response, $msg, $hhead);
}

sub statd_query_machines {
   my($mach, $key, $query) = @_;
   my($msg) = "";
   
   my (@mach);
   if ($mach eq "_sum") {
      @mach = @machines;
   } else {
      @mach = $mach;
   }
   
   my (%h) = ();
   my (%d) = ();
   foreach $machine (@mach) {
      my ($pid) = statd_query_machine($machine, $key, $query);
      if ($pid > 0) {
         $h{$pid} = $machine;
         last if ($key eq "R"); # Only want to query one machine....
      } else {
         $msg .= "$machine: Could not fork query\n";
      }
   }
   while (scalar(%h)) {
      my ($pid) = wait;
      if ($pid > 0 && defined $h{$pid}) {
         if (!open IN, "/tmp/statd.$pid.dat") {
             $msg .= "$h{$pid}: Statd query did not complete\n";
         } else {
            my ($data);
            read IN, $data, (-s "/tmp/statd.$pid.dat");
            close IN;
            my ($stat, $rest) = split "\n", $data, 2;
            if ($stat eq "OK") {
               $d{$h{$pid}} = $rest;
            } else {
               $msg .= "$h{$pid}: $stat\n";
            }
            unlink "/tmp/statd.$pid.dat";
         }
         delete $h{$pid};
      }
   }
   
   if (scalar (keys %d) == 0) {
      return ("", $msg);
   }
   
   my ($response) = "";
   
   # combine response
   foreach $m (keys %d) {
      my ($h, $b) = split "\n", $d{$m}, 2;
      $d{$m} = $b;
      $response = $h if (!$response);
      
      if ($response ne $h) {
         # someone's not got it right....
         delete $d{$m};
         $msg .= "$m: Response header doesn't match!\n";
      }
   }
   if (scalar (keys %d) == 1) {
      return ($response."\n".$d{(keys %d)[0]}, $msg);
   }

   $response .= "\n";
   while ($d{(keys %d)[0]}) {
      my (@data) = ();
      my ($max) = 0;
      my ($avg) = 0;
      my ($cur) = 0;

      foreach $m (keys %d) {
         my ($d1, $d2, $b) = split "\n", $d{$m}, 3;
         my (@_data) = map $_+0, split " ", $d1;
         my ($_max, $_avg, $_cur) = split / /, $d2;

         $d{$m} = $b;

         next if (!defined $_cur);

         # We're not magical enough to do this with $max too...
         $avg += $_avg; $cur += $_cur;
         
         if (!@data) {
            @data = @_data;
         } else {
            for ($i = 0; $i <= $#data; $i++) {
               $data[$i] = ($data[$i]==-1)?
                  $_data[$i] : ($data[$i]+(($_data[$i]==-1)?0:$_data[$i]));
            }
         }
      }
      for ($i = 0; $i <= $#data; $i++) {
         $max = ($data[$i]>$max)?$data[$i]:$max;
      }
      $response .= join " ", @data;
      $response .= "\n$max $avg $cur\n";
   }
   return ($response, $msg);
}

sub timetogo {
   exit 0;
}

sub statd_query_machine {
   my($machine, $key, $query) = @_;

   my($pid) = fork;
   
   return (defined $pid)?$pid:-1 if ((defined $pid) && ($pid != 0));

   # In child

   ($host, $port) = split /:/, $machine;
   $command = "statd ".statd_encode($key, "statd.cgi.$$\t$query");
   
   $response = WebAdmin::command( $host, $port, $command, $err );
   if( !defined($response) ) {
       # Command failed, fake the response with embedded error
       $response = "statd.cgi.$$\n$err";
   }

   # Decode response
   ($uniqueid, $stat, $data) = split /\n/, $response, 3;

   my ($out);

   if ($uniqueid ne "statd.cgi.$$") {
      $out = "Response did not match query\n";
   } elsif ($stat ne "OK") {
      $out = $stat."\n";
   } else {
      $out = "OK\n".$data;
   }

   open OUT, ">/tmp/statd.$$.dat"; print OUT $out; close OUT;

   exit 0;
}

sub statd_time {
   my($time) = @_;
   my($sec, $min, $hour, $mday, $mon, $year, $from, $step, $npts);

   if ($time eq "24hours") {
      ($sec,$min,$hour,$mday,$mon,$year)=localtime($now-86400);
      $min = int($min/5)*5;
      $from = sprintf("%4d%02d%02d%02d%02d",$year+1900, $mon+1, $mday, $hour, $min);
      $step = 5; $npts = 288;
   } else {
      ($sec,$min,$hour,$mday,$mon,$year)=localtime($now-7*86400);
      $min = int($min/30)*30;
      $from = sprintf("%4d%02d%02d%02d%02d",$year+1900, $mon+1, $mday, $hour, $min);
      $step = 30; $npts = 336;
   }
   return "$from $step $npts";   
}

sub statd_encode {
   my ($key, $msg) = @_;
   
   return sprintf("%s%04x\t%s", $key, length($msg)+length($key)+5, $msg);
}

sub plot_graph {
   my ($response, $title, $value, $time, $query, @val) = @_;

   my ($params, $data) = split /\n/, $response, 2;
   my ($nchans, $npts, $step, $sdate, $sday) = split /\s+/, $params;
   my ($file) = "./cache/graph.$hash.gif";

   if ($value eq "bytes") {
      $ytitle = "Bytes per second";
      $scale = 1/$step/60;
   } else {
      $ytitle = "Hits per minute";
      $scale = 1/$step;
   }
   
   my ($date);
   $date = (($time eq "24hours")?"24 hours ending ":"7 days ending ").localtime;
   $date =~ s/:[0-9][0-9] / /;
   
   # Clean up files > 0.001 days (~1.5 minutes) old
   unlink grep {-M $_>0.001} <./cache/*>;
   
   open GRAPHER, "| $ENV{'ZEUSHOME'}/webadmin/bin/graph.line > $file";

   print GRAPHER "$npts $nchans $scale\n";
   print GRAPHER (join " ", (0..$npts-1))."\n";
   print GRAPHER "$title\n";
   print GRAPHER "$date\n";
   print GRAPHER "$ytitle\n";

   # X axis
   print GRAPHER "user\n";
   my(@aticks) = (); my(@alabels) = ();
   if ($npts*$step < 60*48) {
      # Ticks and labels at 2 hr intervals
      $sdate=~/[0-9]{8}([0-9]{2})([0-9]{2})/; 
      $m = 60*$1+$2;
      foreach $i (0..$npts-1) {
         if (($m % 120) == 0) {
            push @aticks, $i; 
            push @alabels, $i." \"".($m/60)."\"";
         }
         $m = ($m+$step)%1440;
      }
   } else {
      # Ticks and labels at 1 day intervals
      $sdate=~/[0-9]{8}([0-9]{2})([0-9]{2})/; 
      $m = 60*$1+$2;
      foreach $i (0..$npts-1) {
         if (($m % 1440) == 0) {
            push @aticks, $i; $sday++;
         }
         if (($m % 1440) == 720) {
            push @alabels, "$i \"$days[$sday%7]\"";
         }
         $m = $m+$step;
      }
   }
   print GRAPHER "0 ".($npts-1)."\n";
   print GRAPHER (join " ", scalar(@aticks),  @aticks)."\n";
   print GRAPHER (join " ", scalar(@alabels), @alabels)."\n";

   # Y axis
   print GRAPHER "auto\n";

   my ($html) = "<table cellspacing=4 cellpadding=0 border=0>
<tr><td colspan=8><b>$ytitle</b></td></tr>\n";

   # data
   foreach $c (0..$nchans-1) {
      my ($cdata, $cparams, $datat) = split /\n/, $data, 3; $data = $datat;
      my ($max,$avg,$cur) = split /\s+/, $cparams;
      my ($label, $col, $url);

      $avg = sprintf("%.2f", ($avg<0)?0:$avg*$scale);
      $max = sprintf("%.2f", ($max<0)?0:$max*$scale);
      $cur = sprintf("%.2f", ($cur<0)?0:$cur*$scale);
      $label = prettify(prettify_machinename($val[$c]));
      ($url = $query) =~ s/!!!!/$val[$c]/e;
      $col = $cols[$c%($#cols+1)];
   
      $html .= "
<tr>
<td bgcolor=\"#$col\"><font color=\"#$col\"><sup>___</sup></font></td>
<td><a href=\"graph.cgi?$url\"><b>$label</b></a>&nbsp;&nbsp;</td>
<td>Max:</td>     <td align=right><b>$max</b>;&nbsp;&nbsp;</td>
<td>Average:</td> <td align=right><b>$avg</b>;&nbsp;&nbsp;</td>
<td>Current:</td> <td align=right><b>$cur</b>.</td>
</tr>
";
      print GRAPHER "$col\n$cdata\n";
   }
   $html .= "</table>\n";
   close GRAPHER;
   return ($file, $html);
}

sub print_html {
   my ($html) = @_;
   # search for embedded title tag, if any
   $html =~ s@<!title>(.*)<!/title>@$1@;
   my $title = $1;
   print <<EOM;
Content-type: text/html

<html>
<head><title>$title</title></head>
<body bgcolor="white">
$html
</body>
</html>
EOM
}

sub hash {
   my ($str) = @_;
   
   my ($hash) = 0;
   
   foreach $c (split //, $str) {
	$hash = ($hash<<5)^($hash>>27)^ord($c);
   }
   
   return (unpack('H8', pack("i", $hash)));
}

sub url_esc {
    my ($toencode) = @_;
    $toencode=~s/([^a-zA-Z0-9_\-.])/uc sprintf("%%%02x",ord($1))/eg;
    return $toencode;
}

sub Error {
   my ($msg) = @_;
   my ($file) = "./cache/error.$$.gif";
   my ($html);
   
   open GRAPHER, "| $ENV{'ZEUSHOME'}/webadmin/bin/graph.err > $file";
   print GRAPHER $msg;
   close GRAPHER;
   
   $html = "<img src=\"$file\">\n";
   
   print_html($html);
   exit 0;
}





sub prettify {
   my ($s) = @_;
   
   $s = "Each Machine" if ($s eq "_each");
   $s = "Total"        if ($s eq "_sum");
   $s = ""             if ($s eq "_line");
   $s = "Top $1"       if ($s =~ /^_top([0-9]+)$/);

   return $s;
}

sub prettify_2 {
   my ($s) = @_;
   
   $s = "*" if ($s eq "_sum");
   $s = prettify($s);
   
   return $s;
}


sub prettify_3 {
   my ($s) = @_;

   $s = prettify_machinename( $s );
   $s = prettify_2($s);
   
   return $s;
}

# Given a machine name "localhost:9080", returns a 'nice' version of the
# name that doesn't contain the port number, unless there are two webservers
# installed on the same machine.
# Doesn't do anything if its not a webserver control port.
#
# Uses the global array @machines to determine this...
sub prettify_machinename {
    my ($s) = @_;
    return $s if( ! $s =~ /:\d+$/ );
    my ($host,$port) = split( ':' , $s );
    return ( scalar(grep (/^$host:/, @machines)) == 1 ) ? $host : $s;
}

sub parse_form_data {
   local (*FORM_DATA) = @_;

   local ($request_method, $query_string, $key_value, $key, $value);

   $request_method = $ENV{'REQUEST_METHOD'};

   if ($request_method eq "GET") {
      $query_string = $ENV{'QUERY_STRING'};
   } elsif ($request_method eq "POST") {
      $query_string = "";
      read STDIN, $query_string, $ENV{'CONTENT_LENGTH'};
   }

   foreach $key_value (split /&/, $query_string) {
      ($key, $value) = split /=/, $key_value, 2;
      $value =~ tr/+/ /;
      $value =~ s/%([\dA-Fa-f][\dA-Fa-f])/pack("C", hex($1))/eg;
      if (defined $FORM_DATA{$key}) {
         $FORM_DATA{$key} = join " ", $FORM_DATA{$key}, $value;
      } else {
         $FORM_DATA{$key} = $value;
      }
   }
   return 0;
}              
